@@ -10,7 +10,7 @@
 .onLoad <- function(libname, pkgname)
 {
   # Load libswipl.dylib under macOS
-  if(.Platform$OS.type == "unix" & R.version$os != "linux-gnu")
+  if(.Platform$OS.type == "unix")
   {
     # Find folder like x86_64-linux
     fp <- file.path(libname, pkgname, "swipl", "lib", "swipl", "lib")
@@ -20,7 +20,10 @@
     folder <- dir(fp, pattern=arch, full.names=TRUE)
 	
     # Preload libswipl.dll
-    dyn.load(file.path(folder, "libswipl.dylib")) # macOS
+    if(R.version$os == "linux-gnu")
+      dyn.load(file.path(folder, paste("libswipl", .Platform$dynlib.ext, sep="")))
+    else
+      dyn.load(file.path(folder, "libswipl.dylib")) # macOS
   }
 	
   if(.Platform$OS.type == "windows")
@@ -52,14 +55,17 @@
   # See .onLoad for details
   library.dynam.unload("rolog", libpath=libpath)
 
-  if(.Platform$OS.type == "unix" & R.version$os != "linux-gnu")
+  if(.Platform$OS.type == "unix")
   {
     fp <- file.path(libpath, "swipl", "lib", "swipl", "lib")
     arch <- R.version$arch
     if(arch == 'aarch64')
       arch <- 'arm64'
     folder <- dir(fp, pattern=arch, full.names=TRUE)
-    dyn.unload(file.path(folder, "libswipl.dylib"))
+    if(R.version$os == "linux-gnu")
+      dyn.unload(file.path(folder, paste("libswipl", .Platform$dynlib.ext, sep="")))
+    else
+      dyn.unload(file.path(folder, "libswipl.dylib")) # macos
   }
 	
   if(.Platform$OS.type == "windows")
