@@ -1781,7 +1781,8 @@ do_create_process_fork(p_options *info, create_method method)
 // #pragma GCC diagnostic push
 // #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
   if ( method == PCREATE_VFORK )
-    pid = vfork();				/* As long as it is there we use it */
+    // pid = vfork();				/* As long as it is there we use it */
+    pid = fork();                               /* MG: suppress a CRAN warning on macOS */
   else
     pid = fork();
   if ( pid != 0 )				/* parent */
